// Klasa za liste geometrijskih figura (Lista).

#ifndef _lista2_h_
#define _lista2_h_

#include "zbirka.h"

class Lista: public Zbirka {
  friend class L_iter;                       // Iterator za liste.
  struct Elem {                              // Element liste:
    Figura* fig;                             //   - figura u elementu
    Elem* sled;                              //   - sledei element
    Elem(Figura* f, Elem* s=nullptr)         //   - konstruktor elementa
      { fig = f; sled = s; }
  };
  Elem *prvi, *posl;                         // Prvi i poslednji element.
  int  duz;                                  // Broj elemenata liste.
  void kopiraj(const Zbirka& z);             // Kopiranje u listu.
  void premesti(Zbirka& z) {                 // Premetanje u listu.
    Lista& lst = dynamic_cast<Lista&>(z);
    prvi = lst.prvi; posl = lst.posl; duz = lst.duz;
    lst.prvi = lst.posl = nullptr;
  }
  void brisi();                              // Oslobaanje memorije.
public:
  Lista() { prvi = posl = nullptr; duz = 0; }// Konstruktor prazne liste.
  Lista(const Lista& lst) { kopiraj(lst); }  // Kopirajui konstruktor.
  Lista(Lista&& lst) { premesti(lst); }      // Premetajui konstruktor.
  Lista& operator=(const Lista& lst) {       // Kopirajua dodela
    if (this != &lst) { brisi(); kopiraj(lst); }  //    vrednosti.
    return *this;
  }
  Lista& operator=(Lista&& lst) {            // Premetajua dodela
    if (this != &lst) { brisi(); premesti(lst); } //    vrednosti.
    return *this;
  }
  int vel() const { return duz; }            // Broj elemenata liste.
  Lista& operator+=(Figura* fig) {           // Dodavanje figure.
    posl = (!prvi ? prvi : posl->sled) = new Elem(fig);
    duz++;
    return *this;
  }
  Lista& operator+=(const Figura& fig)
    { return *this += fig.kopija(); }
  Lista& operator+=(Figura&& fig)
    { return *this += move(fig).kopija(); }
  Figura*& operator[](int ind);              // Dohvatanje figure.
  const Figura* operator[](int ind) const
    { return const_cast<Lista&>(*this)[ind]; }
  Lista& operator~()                         // Pranjenje liste.
    { brisi(); return *this;}
  Lista* kopija() const&                     // Stvaranje kopije liste.
    { return new Lista(*this); }
  Lista* kopija() && { return new Lista(move(*this)); }
  Iter* iter();                              // Stvaranje iteratora.
  const Iter* iter() const;
};

#endif
